const axios = require("axios");
const WebSocket = require("ws");
const prompt = require("prompt-sync")({ sigint: true });

const color = require("./colorlog.js");
color.colors = [
  {
    name: "success",
    color: "32",
  },
  {
    name: "info",
    color: "34",
  },
  {
    name: "danger",
    color: "31",
  },
  {
    name: "warning",
    color: "33",
  },
  {
    name: "sub",
    color: "30",
  },
  {
    name: "special",
    color: "35",
  },
];

/*requires*/

const standardUrl = "https://dev-nakama.winterpixel.io/";
const homeUrl = "wss://dev-nakama.winterpixel.io/ws?lang=en&status=true&token=";

async function sendReq(data, uri, auth, method) {
  data = JSON.stringify(data);
  return await axios({
    method: method,
    url: standardUrl + uri,
    data: data,
    headers: {
      Authorization: auth,
    },
  })
    .then((res) => res.data)
    .catch((err) => color.log("[§[d]Request failed.§§]" + err));
}

async function getToken(email, password) {
  let requestData = {
    email: email,
    password: password,
    vars: {
      client_version: "94387677",
    },
  };

  return await sendReq(
    requestData,
    "v2/account/authenticate/email?create=false&=",
    "Basic OTAyaXViZGFmOWgyZTlocXBldzBmYjlhZWIzOTo=",
    "post",
  ).then((res) => res.token);
}

function getRandomArbitrary(min, max) {
  return Math.random() * (max - min) + min;
}

async function setTank(bearer, user) {
  //
  let additionalColors = [
    "red",
    "mint",
    "olive",
    "purple",
    "orange",
    "gold",
    "black",
    "blue",
    "yellow",
    "brown",
    "pink",
    "green",
    "gray",
    "white",
  ];
  let pattern = `${additionalColors[0]}_.+`;

  additionalColors.forEach((color) => (pattern += `|${color}_.+`));

  pattern = new RegExp(pattern, "gm");
  //
  let userTankAwards = user.awards.filter(
    (string) => string.match(pattern) !== null,
  ); // get tanks
  let userTanks = userTankAwards
    .map((tankName) => {
      let underscoreSplit = tankName.split("_");
      let combined = underscoreSplit[1];
      for (let i = 1; i < underscoreSplit.length - 1; i++) {
        if (underscoreSplit[i + 1]) combined += `_${underscoreSplit[i + 1]}`;
      }
      return combined;
    }) // get unique name
    .filter((name, index, arr) => arr.indexOf(name) === index); // remove duplicates

  let randomSkin =
    userTanks[Math.floor(getRandomArbitrary(0, userTanks.length - 1))];
  let availableColors = userTankAwards
    .filter((name) => name.includes(randomSkin))
    .map((item) => item.split("_")[0]);
  let randomColor =
    availableColors[
      Math.floor(getRandomArbitrary(0, availableColors.length - 1))
    ];
  let randomTank = `${randomColor}_${randomSkin}`;

  let requestData = { skin: randomTank };

  await sendReq(
    JSON.stringify(requestData),
    "v2/rpc/tankkings_set_skin",
    bearer,
    "post",
  ).then((res) => {
    if (!res) return color.log(`[§[w]Failed to set tank.§§]`);
    color.log(`[§[sp]Changed tank to ${randomTank}!§§]`);
  });
}

(async () => {
  try {
    //
    color.log("[§[i]Getting token§§]");
    let email = prompt("Account Email: ");
    let password = prompt("Account Password: ");
    let token = await getToken(email, password);
    let bearer = "Bearer " + token;

    color.log("[§[i]Fetching user info§§]");
    let userId = await sendReq(null, "v2/account", bearer, "get").then(
      (res) => res.user.id,
    );

    let userData = { user_id: userId };
    let user = await sendReq(
      JSON.stringify(userData),
      "v2/rpc/rpc_get_player_profile",
      bearer,
      "post",
    ).then((res) => JSON.parse(res.payload));

    color.log("[§[i]Starting websocket§§]");
    let homeSocket = new WebSocket(homeUrl + token);
    homeSocket.onmessage = async (event) => {
      let eventData = JSON.parse(event.data);
      if (
        eventData.notifications &&
        eventData.notifications.notifications[0].subject === "stats"
      )
        await setTank(bearer, user);
    };
    //
  } catch (e) {
    color.log(`§[d]${e}`);
  }
})();
